/*
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999-2003 Happy <ljm AT ecs.soton.ac.uk>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/


//#include <iostream.h>

#include <stdlib.h>

//#include <assert.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qmultilineedit.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qpushbutton.h>
#include <qsimplerichtext.h>
#include <qspinbox.h>
#include <qtextstream.h>
#include <qtextview.h>
#include <qvbox.h>
#include <qdir.h>
#include <qarray.h>
#include <qwidget.h>

#include "weeport.hh"

/*--------------------------------------------------------------------
  Class : Weeport
  -------------------------------------------------------------------*/

/*====================================================================
  Constructor (Weeport)                                              
  --------------------------------------------------------------------
  Parameters   : None.
  Return Vals  : initialises all the widgets and the info object
  Processing   : Just sets of geoetry and settings for each of the
                 widgets that are displaying in the report widget. The
		 locations are set manually. Also, sets the info object
		 to be null and calls the change method to create a
		 report for the current week. The layout is now handled
		 by the use of a gridlayout object. This means there is 
		 automatic resizing of all the widgets. Finally, the
		 individual widgets are restricted in size. this makes
		 the report data widget strech more than the others.
		 yes, i know there are lots of magic numbers in here.
  Assumptions  : The widget is designed at the moment to be a specific
                 size and cannot be changed.
  Dependencies : QT objects
======================================================================*/
Weeport::Weeport() 
    : QWidget(0, "weeport")
{

    QVBoxLayout *vbox = new QVBoxLayout( this );
    setCaption("KTT: Report");
    QMenuBar *menu = new QMenuBar( this, "mb_menu" );
    QPopupMenu *date = new QPopupMenu();
    date->insertItem( "This week", this, SLOT(thisWeek()) );
    date->insertItem( "Last week", this, SLOT(lastWeek()) );
    date->insertSeparator( );
    date->insertItem( "Select ...", this, SLOT(chooseWeek()) );
    item = new QPopupMenu();
    item->setCheckable( true );
    QPopupMenu *otha = new QPopupMenu();
    otha->insertItem( "Print ...", this, SLOT(printReport()) );
    menu->insertItem( "Date",  date );
    menu->insertItem( "View", item );
    menu->insertItem( "Other", otha );
    disp = new Weedisplay( this, "wdy_disp" );
    disp->show();
    vbox->setMenuBar( menu );
    vbox->addWidget( disp );

    vbox->activate();

    setBackgroundColor( Qt::white );

    thedate = QDate::currentDate( );

    stuff = 0;
#ifdef DESKTOP_VERSION
    show();
#else
    showMaximized();
#endif
    thisWeek();

}

/*====================================================================
  isWanted(QString name)                                          bool
  --------------------------------------------------------------------
  Parameters   : name - the name of the project to examine
  Return Vals  : true - this project should be in report
                 false - this project shouldnt be in report
  Processing   : Basically, this just goes through all the items in the
                 project list box. If project `name' is found to be
		 selected then it returns true. otherwise it returns 
		 false.
  Assumptions  : None.
  Side Effects : None.
  Dependencies : None.
======================================================================*/
bool Weeport::isWanted(QString name) 
{
    
    bool wanted = false;

    for (int i=0;i<mdata.count() && !wanted;i++) {
	if ( item->isItemChecked( 100*(i+1) ) ) {
	    for (int j=0;j<mdata.at(i)->pnames.count();j++) {
		if ( mdata.at(i)->pnames[j]==name && 
		     item->isItemChecked( 100*(i+1)+j+1 ) ) {
		    wanted = true;
		    break;
		}
	    }
	}
    }
    
    return wanted;

}

/*====================================================================
  updateFromMenu(int id)                                          void
  --------------------------------------------------------------------
  Parameters   : id - meu which has been selected
  Return Vals  : none
  Processing   : updates the report given that a particular menu item
                 in 'View' has been selected. If a category is selected
		 then all the items in that category are set to the state
		 of the category. otherwise the state of the item is
		 toggled. a category has an id of n*100 and a project has
		 an id of n*100+m+1. Once toggled a new display is
		 generated.
  Assumptions  : None.
  Side Effects : changes the stae of items in the view menu and the
                 report displayed on the screen.
  Dependencies : Weedisplay
======================================================================*/
void Weeport::updateFromMenu(int id) 
{

    if ( item->isItemChecked( id ) )
	item->setItemChecked( id, false );
    else
	item->setItemChecked( id, true );

    if (id%100==0) {

	bool newcheck;
	if ( item->isItemChecked( id ) )
	    newcheck = true;
	else
	    newcheck = false;
	
	int i = id/100-1;
	for (int j=0;j<mdata.at(i)->pnames.count();j++) {
	    item->setItemChecked( 100*(i+1)+j+1, newcheck );
	}
    } else {
	int i = 100*(id/100);
	if ( !item->isItemChecked( i ) )
	    item->setItemChecked( i, true );
    }

    QString sdisp = extractData();

    disp->setText( sdisp, this->font() );
    // disp->setFixedSize( disp->sizeHint() );

}

/*====================================================================
  printReport( )                                                  void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : uses the inbuilt QT printing class. A QPrinter is very
                 similar to a QPainter. initially the printer info is 
		 found via a dialog box (setup). after this, the printer
		 is painted to display the report to a printer.
  Assumptions  : QT printer object is working
  Side Effects : generates a printout
  Dependencies : QPrinter, QSimpleRichText, QPaintDeviceMetrics
======================================================================*/
void Weeport::printReport()
{
#ifdef DESKTOP_VERSION
    QPrinter printer;

    if ( !printer.setup( this ) ) 
	return;
    
    QPainter p( &printer );
    // find dimensions of page as told by printer
    QPaintDeviceMetrics metrics(p.device());
    int dpix = metrics.logicalDpiX();
    int dpiy = metrics.logicalDpiY();
    // margin = 2.54 cm = 72pt
    const int margin = 72;
    // define a rectangle the size of a page
    QRect body(0, margin*dpiy/72,
	       metrics.width(),
	       metrics.height() );
    // create the report
    QSimpleRichText richText( extractData(), QFont() );
    richText.setWidth( &p, body.width() );
    QRect view( body );
    // print a page at a time of the report
    int page = 1;
    do {
      richText.draw( &p, body.left(), body.top(), view, colorGroup() );
      view.moveBy( 0, body.height() );
      p.translate( 0 , -body.height() );
      // print page numbers commented out
//       p.drawText(view.right()-p.fontMetrics().width(QString::number(page)),
// 		  view.bottom() + p.fontMetrics().ascent() + 5, 
// 		  QString::number(page) );
      if ( view.top()  >= richText.height() )
	break;
      printer.newPage();
      page++;
    } while (TRUE);
#else
	QMessageBox::warning(0, "KTimeTracker: Sorry!",
	   QString("Printing only supported\non desktop version!"), "OK");
#endif
    
}

/*====================================================================
  thisWeek( )                                                     void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : finds todays date and uses this to update the report.
  Assumptions  : none.
  Side Effects : changes value of class variable `thedate'
  Dependencies : QDate, updateFromDate
======================================================================*/
void Weeport::thisWeek( )
{

    thedate = QDate::currentDate( );
    
    updateFromDate( );

}

/*====================================================================
  lastWeek( )                                                     void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : finds the date a week ago and uses this to update the 
                 report.
  Assumptions  : none.
  Side Effects : changes value of class variable `thedate'
  Dependencies : QDate, updateFromDate
======================================================================*/
void Weeport::lastWeek( )
{

    QDate date = QDate::currentDate( );

    thedate = date.addDays(-7);

    updateFromDate( );

}

/*====================================================================
  lastWeek( )                                                     void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : uses a dialog to input an arbitrary date. This date is
                 used to compute the report for the week.
  Assumptions  : none.
  Side Effects : changes value of class variable `thedate'
  Dependencies : QDate, updateFromDate, Weedate
======================================================================*/
void Weeport::chooseWeek( ) 
{
    
    Weedate wd;
    
    if ( wd.exec() ) {
	thedate = wd.getDate( );
	updateFromDate( );
    }
    
}

/*====================================================================
  toHrsMins(int howlong)                                       QString
  --------------------------------------------------------------------
  Parameters   : howlong - a time in minutes
  Return Vals  : a string of format `hh:mm' that shows the hours/mins
                 for the current value
  Processing   : Just divide the number by 60 to calc the hrs and
                 perform mod 60 for the minutes. this is then put into
		 a string via sprintf.
  Assumptions  : None.
  Side Effects : None.
  Dependencies : QString
======================================================================*/ 
QString Weeport::toHrsMins(int howlong) 
{
    
    QString s;
    int h,m;
    
    h = howlong/60;
    m = howlong%60;
    
    s.sprintf("%d:%2.2d",h,m);
    
    return s;
    
}

/*====================================================================
  extractData()                                                QString
  --------------------------------------------------------------------
  Parameters   : None.
  Return Vals  : None.
  Processing   : The purpose of this routine is to generate and display
                 report info on the selected projects. An array is used
		 store the current totals for each day (and one extra
		 for the totals). Initially a vector of integers is sorted
		 to produce a list of indices for the data sorted
		 alphabetically by category. A loop then goes through 
		 each project in turn excluding unwanted projects. A 
		 string is used to store all the outputted info and this 
		 is added to in the loop. finally this is returned for 
		 use by other methods.
  Assumptions  : there is a valid weeinfo object. the qstring can store
                 all of the info needed to be displayed. the length of
		 the project name is less than 16 chars long. the sort
		 is performed assuming the '>' works as expected for
		 strings.
  Side Effects : Clears and redisplays the multilineedit box
  Dependencies : class weeinfo to get data
                 toHrsMins to display right output data
======================================================================*/
QString Weeport::extractData() 
{
    int store[8];
    
    QString sout;
    QString df = "<td align=right>";
    QDate sdate = thedate.addDays(1-thedate.dayOfWeek()); // start date
    QDate edate = thedate.addDays(7-thedate.dayOfWeek()); // end date

    sout = "<qt><p>From " + sdate.toString( ) + " to " + 
	edate.toString( ) + ":</p>";

    for (int j=0;j<8;j++) {
	store[j] = 0;
    }

    sout += "<table><tr><th>Category</th><th>Item</th>";
    
    sout += "<th>Mon</th><th>Tue</th><th>Wed</th><th>Thur</th><th>Fri</th><th>Sat</th><th>Sun</th><th>Total</th></tr>";    

    // create a sorted list by the category name
    QArray <int> order (stuff->numProjects());
    for (int j=0; j<stuff->numProjects(); j++)
        order[j] = j;
    for (int j=0; j<stuff->numProjects(); j++) {
        QString cj = stuff->getCategoryName(j).lower();
        for (int i=j+1; i<stuff->numProjects(); i++) {
            QString ci = stuff->getCategoryName(i).lower();
            if ( ci<cj ) {
                int temp = order[j];
                order[j] = order[i];
                order[i] = temp;
                break;
            }
        }
    }

    for (int j=0; j<stuff->numProjects(); j++) {
      int k = order[j];
      QString s = stuff->getProjectName(k);
      QString c = stuff->getCategoryName(k);
      if ( isWanted(s) ) {
	sout += "<tr><td>" + c + "</td>";
	sout += "<td>" + s + "</td>";
	int ptot = 0;
	for (int i=0; i<7; i++) {
	  sout += df + toHrsMins(stuff->getProjectData(k,i)) + "</td>";
	  store[i] += stuff->getProjectData(k,i);
	  ptot += stuff->getProjectData(k,i);
	}
	store[7] += ptot;
	sout += df + toHrsMins(ptot) + "</td></tr>";
      }
    }
    
    sout += "<tr>";
    sout += "<td></td><td>total</td>";
    for (int j=0; j<8; j++) {
	sout += df + toHrsMins(store[j]) + "</td>";
    }
    sout += "</tr></table></qt>";  
    
    return sout;
  
}

/*====================================================================
  updateFromDate( )                                               void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : computes and displays a new report when the date has
                 changed. the data is first computed. then the menus 
		 are populated and finally the richtext widget is 
		 draw to the window.
  Assumptions  : none.
  Side Effects : changes value of class variable `stuff'
  Dependencies : Weeinfo, extractData( ), Weedisplay
======================================================================*/
void Weeport::updateFromDate( )
{

    if ( stuff!=NULL )
	delete stuff;

    stuff = new Weeinfo( thedate );
    doMenus();

    QString sdisp = extractData( );

    disp->setText( sdisp, font() );
    // disp->setFixedSize( disp->sizeHint() );

}

/*====================================================================
  doMenus( )                                                      void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : computes a whole series of new menus for the view menu
                 when the user has changed the date. 
  Assumptions  : none.
  Side Effects : changes the view menus, mdata (menu data structure)
  Dependencies : mdata, QPopupMenu
======================================================================*/
void Weeport::doMenus( )
{

    // remove existing connections for categories
    disconnect( item, 0, this, 0 );
    // remove old menus ...
    item->clear();
    mdata.setAutoDelete( true );
    // remove menu data structure
    mdata.clear();
   
    
    // create new menus
    for (int i=0;i<stuff->numProjects();i++) {
	
	QString s = stuff->getProjectName(i);
	QString c = stuff->getCategoryName(i);

	if ( mdata.count()==0) {
	   MData *m = new MData;
	   m->cname = c;
	   m->pnames.append( s );
	   mdata.append( m ); 
	}
	else {

	    int found = -1;
	    for (int j=0;j<mdata.count();j++) {
            if ( mdata.at(j)->cname==c ) {
		    found = j;
		    break;
		}
	    }
	    
	    if (found==-1) {
		MData *m = new MData;
		m->cname = c;
		m->pnames.append( s );
		mdata.append( m );
	    }
	    else {
            mdata.at(found)->pnames.append( s );
	    }
	}	
    }

    // fill menus for categories
    for (int i=0;i<mdata.count();i++) {
	item->insertItem( mdata.at(i)->cname, 100*(i+1) );
	item->setItemChecked( 100*(i+1), true );
    }
    item->insertSeparator();
    // ... and for projects themselves
    for (int i=0;i<mdata.count();i++) {
	QPopupMenu *p = new QPopupMenu( this, mdata.at(i)->cname );
	p->setCheckable( true );

	for (int j=0;j<mdata.at(i)->pnames.count();j++) {
	    p->insertItem( mdata.at(i)->pnames[j], 100*(i+1)+j+1 );
	    p->setItemChecked( 100*(i+1)+j+1, true );
	}
	
	item->insertItem( mdata.at(i)->cname, p );
	connect( p, SIGNAL(activated(int)), 
		 this, SLOT(updateFromMenu(int)) );
    }    
    connect( item, SIGNAL(activated(int)), 
	     this, SLOT(updateFromMenu(int)) );

  
}


/*----------------------------------------------------------------------
  Class : Weeinfo
  -------------------------------------------------------------------*/

/*====================================================================
  Constructor (Weeinfo)                                              
  --------------------------------------------------------------------
  Parameters   : date - the date for the start of the period of which
                        to calculate the data for
  Return Vals  : updates class vector storing info on projects.
  Processing   : First using QDate calculate the start and end date of
                 the week being examined. Then a loop is performed for
		 each of the days in that range. For each day the correct
		 file is opened (in the $PTIMETRACKER directory). All
		 lines are then read in turn and valid ones are used to
		 add data to the vector (using addData). finally the
		 file is closed and then next file examined. A check
		 is made to see if the file exists.
  Assumptions  : None.
  Dependencies : dateToFile - converts a QDate to `YYYY-MM-DD'
                 addData - the current project polus time to the vector
		 QDate, QString, QFile, QTextStream
======================================================================*/
Weeinfo::Weeinfo(QDate date) 
{

    QDate sdate = date.addDays(1-date.dayOfWeek()); // start date
    QDate edate = date.addDays(7-date.dayOfWeek()); // end date

    QString dir;
    QString dirvar = getenv("PTIMETRACKERDIR");
    if(dirvar.isEmpty())
	dir = QDir::homeDirPath() + "/kdepim/apps/ktimetracker/";
    else
	dir = dirvar + "/";    
    
    for ( QDate tdate = sdate; tdate<=edate; tdate = tdate.addDays(1) ) {
	
	QString s = dateToFile(tdate);

	QFile f(dir+s);
	if ( f.open(IO_ReadOnly) ) {
	    QTextStream ts(&f);
	    while ( !ts.eof() ) {
		QString sdata = ts.readLine().stripWhiteSpace();
		if ( sdata.contains('#',false)==0 ) {
	
		    addData(sdata,tdate.dayOfWeek());	    
		    
		}
		
	    }
	    
	}
	f.close();

    }
    projs.setAutoDelete( true );
}

/*====================================================================
  Destructor (Weeinfo)
  --------------------------------------------------------------------
  Parameters   : None. 
  Return Vals  : None.
  Processing   : It deletes each of the items in the vector in turn
                 until there are no more.
  Assumptions  : None.
  Dependencies : None.
======================================================================*/
Weeinfo::~Weeinfo() 
{

    
}

/*====================================================================
  numProjects()                                                    int
  --------------------------------------------------------------------
  Parameters   : none.
  Return Vals  : size of vector
  Processing   : just returns the size of the vector
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
int Weeinfo::numProjects() 
{

    return projs.count();
    
}

/*====================================================================
  getProjectName(int i)                                        QString
  --------------------------------------------------------------------
  Parameters   : i - the number of the project to return the name of
  Return Vals  : the name of the project
  Processing   : just returns the name of the project in position i of
                 the vector
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
QString Weeinfo::getProjectName(int i) 
{
    
    return projs.at(i)->proj_name;
    
}

/*====================================================================
  getCategoryName(int i)                                        QString
  --------------------------------------------------------------------
  Parameters   : i - the number of the project to return the name of
  Return Vals  : the name of the project
  Processing   : just returns the category of the project in position i
                 of the vector
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
QString Weeinfo::getCategoryName(int i) 
{
    
    return projs.at(i)->caty_name;
    
}

/*====================================================================
  getProjectData(int i, int day)                                   int
  --------------------------------------------------------------------
  Parameters   : i - a number corresponding to a project
                 day - which day to return data for
  Return Vals  : an integer representing the amount minutes worked
  Processing   : just returns the data for project i. it looks at the
                 array to return the data for the specific day.
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
int Weeinfo::getProjectData(int i, int day) 
{

    return projs.at(i)->proj_spnt[day];

}

/*====================================================================
  dateToFile(QDate date)                                       QString
  --------------------------------------------------------------------
  Parameters   : date - the date to convert
  Return Vals  : a string representing a reformated date
  Processing   : converts from Qdate to `YYYY-MM-DD' which is the
                 format used in the files.
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
QString Weeinfo::dateToFile(QDate date) 
{
    
    QString s;
    s.sprintf("%04d-%02d-%02d", date.year(), date.month(), date.day());

    return s;
    
}

/*====================================================================
  addData(QString s, int day)                                     void
  --------------------------------------------------------------------
  Parameters   : s - a valid line from the data file 
                     ie has format `project hh:mm'
                 day - which day of the week we are examining
  Return Vals  : none.
  Processing   : First extracts the project name and the hours and 
                 minutes. If the projectname is new then it adds it to
		 the end of the vector. The hours and minutes are then
		 coverted to a number just in terms of minutes. this is
		 added to the poject at the appropriate day (in the
		 array).
  Assumptions  : none.
  Side Effects : updates the vector
  Dependencies : none.
======================================================================*/
void Weeinfo::addData(QString s,int day) 
{

    if ( s.isNull() ) 
	return;
    
    int i,j,k; // position of separators
    QString category,project;

    i = s.find(' ',0,false);
    j = s.find(':',0,false);

    if ( (k = s.find('^',0,false))==-1 ) { // bug fix for no category
	category = "unknown";
	project = s.mid(i+1,s.length());
    }
    else {
	category = s.mid(i+1,k-i-1);
	project = s.mid(k+1,s.length()); 
    }
    
    int time = 60*((s.left(j)).toInt()) + ((s.mid(j+1,i-2)).toInt());

    if (projs.count() == 0) {
	PData *first = new PData;
	first->proj_name = project;
	first->caty_name = category;
	for (int i=0;i<7;i++) {
	    first->proj_spnt[i] = 0;
	}
	first->proj_spnt[day-1] = time;
	projs.append(first);
    }
    else {

	int found = -1;
    
	for (int i=0;i<projs.count();i++) {
	    if ( projs.at(i)->proj_name == project ) {
		found = i;
		break;
	    }
	}
	
	if (found!=-1) {
	    projs.at(found)->proj_spnt[day-1] += time;
	} else {
	   PData *other = new PData;
	   other->proj_name = project;
	   other->caty_name = category;
	   for (int i=0;i<7;i++) {
	       other->proj_spnt[i] = 0;
	   }
	   other->proj_spnt[day-1] = time;
	   projs.append(other); 
	}
	
    }
        
}

/*--------------------------------------------------------------------
  Class : Weedate
  -------------------------------------------------------------------*/

/*====================================================================
  Constructor (Weedate)                                              
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : Lays out the dialog box. 
  Assumptions  : None.
  Dependencies : various QWidgets
======================================================================*/
Weedate::Weedate( ) 
    : QDialog( 0, "date selection", true )
{
    
    setCaption( "Select date" );
    setFixedSize( 180,180 );
    
    QHBox *first = new QHBox( this, "first" );
    first->setMargin( 5 );
    first->setSpacing( 5 );
    day = new QSpinBox( 1,31, 1, first, "day" );
    QLabel *l_day = new QLabel( "Day", first );
    
    QHBox *secnd = new QHBox( this, "secnd" );
    secnd->setMargin( 5 );
    secnd->setSpacing( 5 );
    month = new QSpinBox( 1,12, 1, secnd, "month" );
    QLabel *l_month = new QLabel( "Month", secnd );

    QHBox *third = new QHBox( this, "third" );
    third->setMargin( 5 );
    third->setSpacing( 5 );
    year = new QSpinBox( 1990,3000, 1, third, "year" );
    QLabel *l_year = new QLabel( "Year", third );

    QHBox *forth = new QHBox( this, "forth" );
    forth->setMargin( 5 );
    forth->setSpacing( 5 );
    QPushButton *ok = new QPushButton( "OK", forth, "ok" );
    QPushButton *cancel = new QPushButton( "Cancel", forth, "cancel" );

    QVBoxLayout *vb = new QVBoxLayout( this );
    vb->addWidget( first );
    vb->addWidget( secnd );
    vb->addWidget( third );
    vb->addWidget( forth );
    
    QDate today = QDate::currentDate( );

    day->setValue( today.day() );
    month->setValue( today.month() );
    year->setValue( today.year() );

    connect( ok, SIGNAL(clicked()), this, SLOT(accept()) );
    connect( cancel, SIGNAL(clicked()), this, SLOT(reject()) );

}

/*====================================================================
  getDate( )                                                     QDate
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : the date that was entered
  Processing   : simply returns the date in the spin boxes with a small
                 rangecheck on the data.
  Assumptions  : none.
  Side Effects : none.
  Dependencies : none.
======================================================================*/
QDate Weedate::getDate( ) 
{
    QDate tdate = QDate(year->value(), month->value(),1);
    
    if ( day->value()>tdate.daysInMonth() )
	return QDate( year->value(), month->value(), tdate.daysInMonth() );
    else
	return QDate( year->value(), month->value(), day->value() );

}


/*--------------------------------------------------------------------
  Class : Weedisplay
  -------------------------------------------------------------------*/

/*====================================================================
  Constructor (Weedisplay)                                           
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : sets background to cyan and initialises the widget to
                 have no content.
  Assumptions  : None.
  Dependencies : various QWidgets
======================================================================*/
Weedisplay::Weedisplay( QWidget *parent, const char *name )
    : QScrollView( parent, name )
{

    setBackgroundColor( Qt::white );
    data = 0;
    dataPrint = new QTextView( viewport() );
    addChild(dataPrint);
    //setResizePolicy (QScrollView::AutoOneFit );
    viewport()->setBackgroundColor(  Qt::white );
}

/*====================================================================
  setText( )                                                      void
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : none
  Processing   : sets the text in the rich text widget to be the one
                 that is supplied. it formats this in the supplied font.
  Assumptions  : assumes the text is valid rich text
  Side Effects : changes appearance of widget
  Dependencies : QSimpleRichText
======================================================================*/
void Weedisplay::setText( QString text, QFont )
{
    dataPrint->setText(text); 
    dataPrint->setMinimumSize( 640, 480 );


}

/*====================================================================
  sizeHint( )                                                    QSize
  --------------------------------------------------------------------
  Parameters   : none
  Return Vals  : the prefered size of the widget
  Processing   : returns the prefered size of the widget. if there is no
                 content then a dummy size is supplied. this is to allow
		 the widget to be exactly the right size for the content
  Assumptions  : assumes the text is valid rich text
  Side Effects : changes appearance of widget
  Dependencies : QSimpleRichText
======================================================================*/
/*
QSize Weedisplay::sizeHint( )
{
    dataPrint->resize( data->width()+100, data->height()+100 );
    if ( data==NULL )
	return QSize(100,100);

    return QSize( data->width(), data->height() );


}
*/
/*====================================================================
  paintEvent(QPaintEvent *event)                                  void
  --------------------------------------------------------------------
  Parameters   : event - the internal event data provided by QT
  Return Vals  : none
  Processing   : if the widget has some data in it then it will redraw
                 the widget.
  Assumptions  : none
  Side Effects : changes appearance of widget
  Dependencies : QPainter
======================================================================*/
void Weedisplay::paintEvent(QPaintEvent *event)
{
    QScrollView::paintEvent(event);
}









