/* 
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999-2003 Sylvia Wong <s DOT wong AT auckland.ac.nz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/

#define VERSION "0.9.7"

#include "widgets.hh"
#include "sucker.hh"
#include "weeport.hh"

#include <stdlib.h>
//#include <unistd.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qtoolbutton.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qdir.h>
#include <qlayout.h>

#include "icons/close.xpm"
#include "icons/add.xpm"
#include "icons/play.xpm"
#include "icons/pause.xpm"
#include "icons/report.xpm"

#define ID_ALL_VIEWMENU  100
#define ID_UNFILED_VIEWMENU  101

/************************************************************************
   add project dialog
*/

AddDialog::AddDialog(QStringList cList)
    : QDialog(0, "add dialog", true)
{
    setCaption("KTimeTracker: Add Project");

    QPushButton *okBtn = new QPushButton( "OK", this, "okButton" );
    QPushButton *cancelBtn = new QPushButton( "Cancel", this, "cancelButton" );
    connect(okBtn, SIGNAL(clicked()), this, SLOT(accept()) );
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()) );

    input = new QLineEdit( this, "input" );
    input->setMinimumWidth(170);
    input->setFocusPolicy( QWidget::StrongFocus );

    category_cb = new QComboBox( false, this, "category_cb" );
    category_cb->setMinimumWidth(170);
	category_cb->insertStringList( cList);
    category_cb->insertItem("Unfiled");
    category_cb->setCurrentItem(category_cb->count()-1);

    QVBoxLayout *vbox = new QVBoxLayout(this,10,5);
    QGridLayout *grid = new QGridLayout(vbox,2,2,10);
    grid->addWidget(new QLabel("Name", this),0,0);
    grid->addWidget(new QLabel("Category", this),1,0);
    grid->addWidget(input,0,1);
    grid->addWidget(category_cb,1,1);
    QHBoxLayout *hbox = new QHBoxLayout(vbox,10);
    hbox->addWidget(okBtn);
    hbox->addWidget(cancelBtn);
}

int AddDialog::getCategoryInd()
{
    return category_cb->currentItem();
}

/************************************************************************
   add category dialog
*/

CatDialog::CatDialog()
    : QDialog(0, "category dialog", true)
{
    setCaption("KTimeTracker: Add Category");

    QPushButton *okBtn = new QPushButton("OK", this);
    QPushButton *cancelBtn = new QPushButton("Cancel", this);
    connect(okBtn, SIGNAL(clicked()), this, SLOT(accept()) );
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()) );

    input = new QLineEdit( this, "input" );
    input->setMinimumWidth(170);
    input->setFocusPolicy( QWidget::StrongFocus );

    QVBoxLayout *vbox = new QVBoxLayout(this,10,5);
    vbox->addWidget(new QLabel("Category to add",this));
    vbox->addWidget(input);
    QHBoxLayout *hbox = new QHBoxLayout(vbox,10);
    hbox->addWidget(okBtn);
    hbox->addWidget(cancelBtn);
}

/************************************************************************
   delete category dialog
*/
DelCatDialog::DelCatDialog( QStringList cList ) :
    QDialog(0, "delete category dialog", true)
{
    setCaption( "KTimeTracker: Delete Category" );
    QGridLayout *grid = new QGridLayout( this,2,2,10);

    listbox = new QListBox(this);
	listbox->insertStringList( cList);
    grid->addMultiCellWidget( listbox, 0, 0, 0, 1 );
    grid->addRowSpacing( 0, 100 );

    QPushButton *delButt = new QPushButton("Delete", this);
    grid->addWidget( delButt, 1, 0 );
    QPushButton *canButt = new QPushButton("Cancel", this);
    grid->addWidget( canButt, 1, 1 );
    grid->setRowStretch( 0, 4 );
    grid->setRowStretch( 1, 1 );

    connect(delButt, SIGNAL(clicked()), this, SLOT(accept()) );
    connect(canButt, SIGNAL(clicked()), this, SLOT(reject()) );
}


QString DelCatDialog::getChosen()
{
    // for a QListBox if nothing is selected currentItem() == -1
    // according to Qt 2.1 API currentText() is text(currentItem())
    // and text() should return a null string if there are no string
    // at the specfied index. However, due to maybe a bug in Qt 2.1.0,
    // currentText() returns the string at index 0 if 
    // currentItem() < 0. Therefore need to check explicitly
    
    if( listbox->currentItem() < 0 )
	return "";
    else
	return listbox->currentText();
}


/************************************************************************
   edit project dialog
*/
EditDialog::EditDialog(QString pname, int pcat, int time, 
		       QStringList cList)
    : QDialog(0, "edit dialog", true)
{
    setCaption("Edit " + pname);

    category_cb = new QComboBox( false, this, "category_cb" );
	category_cb->insertStringList( cList);
    category_cb->insertItem("Unfiled");
    category_cb->setCurrentItem(pcat);

    int h = time / 60;
    int m = time - h * 60;
    hrbox = new QSpinBox(0, 24, 1, this, "hrbox");
    hrbox->setValue(h);
    minbox = new QSpinBox(0, 60, 5, this, "minbox");
    minbox->setValue(m);

    QPushButton *okBtn = new QPushButton("OK", this);
    QPushButton *cancelBtn = new QPushButton("Cancel", this);
    connect(okBtn, SIGNAL(clicked()), this, SLOT(accept()) );
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()) );

    QVBoxLayout *m_vbox = new QVBoxLayout(this,10,5);

    QGridLayout *grid = new QGridLayout(m_vbox,4,4,5);
    grid->addMultiCellWidget(new QLabel("Project Name:",this),0,0,0,1);
    grid->addMultiCellWidget(new QLabel(pname,this),0,0,2,3);
    grid->addMultiCellWidget(new QLabel("Category:",this),1,1,0,1);
    grid->addMultiCellWidget(category_cb,1,1,2,3);
    grid->addMultiCellWidget(new QLabel("Accumulated Time (Today only):",this),2,2,0,3);
    //grid->addWidget(new QLabel("(Today only)",this),3,0);
    grid->addWidget(hrbox,3,0);
    grid->addWidget(new QLabel("hours",this),3,1);
    grid->addWidget(minbox,3,2);
    grid->addWidget(new QLabel("min",this),3,3);
    
    QHBoxLayout *hbox = new QHBoxLayout(m_vbox,10);
    hbox->addWidget(okBtn);
    hbox->addWidget(cancelBtn);
};

int EditDialog::getTime()
{
    return ( (hrbox->value() * 60) + minbox->value() );
}

int EditDialog::getCategoryInd()
{
    return category_cb->currentItem();
}

/************************************************************************
   preference project dialog
*/
static const char *sel[] = {
    "T project time",
    "T time project",
    "T project",
    "T time",
    "project time",
    "time project",
    "project",
    "time",
    "KTimeTracker: project time",
    "KTimeTrakcer: time project",
    "KTimeTracker: project",
    "KTimeTracker: time"
};

PrefDialog::PrefDialog(int freq, int captiont, int icont,
		       int autostopTime)
    : QDialog(0, "preference dialog", true)
{
    freqbox = new QSpinBox(5, 60, 1, this, "freqbox");
    freqbox->setValue(freq);

    autostop_cb = new QCheckBox("Enable Autostop at", this, "enable_box");
    autostop_hour = new QSpinBox(0, 23, 1, this, "hour_box");
    autostop_min = new QSpinBox(0, 59, 1, this, "min_box");
    if(autostopTime != -1) {
	autostop_cb->setChecked(true);
	int h = autostopTime / 60;
	int m = autostopTime - h * 60;
	autostop_min->setValue(m);
	autostop_hour->setValue(h);
    }

    caption_cb = new QComboBox( false, this, "caption_cb" );
    icon_cb = new QComboBox( false, this, "icon_cb" );
    for(int i=0; i<12; i++) { 
	caption_cb->insertItem( QString(sel[i]) );
	icon_cb->insertItem( QString(sel[i]) );
    }
    caption_cb->setCurrentItem(captiont);
    icon_cb->setCurrentItem(icont);

    QPushButton *okBtn = new QPushButton("OK",this);
    QPushButton *cancelBtn = new QPushButton("Cancel",this);
    connect(okBtn, SIGNAL(clicked()), this, SLOT(accept()) );
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()) );

    setCaption("KTT: Preferences");

    QVBoxLayout *vbox = new QVBoxLayout(this,10,5);
    
    QGridLayout *grid = new QGridLayout(vbox,5,4,5);
    grid->addMultiCellWidget(new QLabel("Autosave freq.:",this),0,0,0,1);
    grid->addWidget(freqbox,0,2);
    grid->addWidget(new QLabel("min",this),0,3);
    //grid->addWidget(new QLabel("Autostop",this),1,0);
    grid->addMultiCellWidget(autostop_cb, 1,1,0,3);
    // grid->addWidget(new QLabel("at",this),2,0);
    grid->addWidget(autostop_hour,2,0);
    grid->addWidget(new QLabel("hour",this),2,1);
    grid->addWidget(autostop_min,2,2);
    grid->addWidget(new QLabel("min",this),2,3);
    grid->addMultiCellWidget(new QLabel("Caption Text:",this),3,3,0,1);
    grid->addMultiCellWidget(caption_cb, 4,4,0,3);
    grid->addMultiCellWidget(new QLabel("Icon Text:",this),5,5,0,1);
    grid->addMultiCellWidget(icon_cb, 6,6,0,3);

    QHBoxLayout *hbox = new QHBoxLayout(vbox,10);
    hbox->addWidget(okBtn);
    hbox->addWidget(cancelBtn);
}

int PrefDialog::getFreq()
{
    return freqbox->value();
}

int PrefDialog::getCaptionText()
{
    return caption_cb->currentItem();
}

int PrefDialog::getIconText()
{
    return icon_cb->currentItem();
}

int PrefDialog::getAutoStopTime()
{
    if(autostop_cb->isChecked())
	return ( (autostop_hour->value() * 60) + autostop_min->value() );
    else
	return -1;
}


//*********************************************************************
Top::Top() : QMainWindow( 0, "KTimeTracker"), currentViewId(ID_ALL_VIEWMENU)
{
    pttDirectory();
    lock();
    readPrefFile();
    int pcur, pheight, pwidth;
    readSessionFile(pwidth, pheight);
    f_catlist = pttDir + "categorylist";
    
    tracker = new Sucker( pttDir, p_savefreq, p_autostopTime,
			  this, "task window" );
    setCentralWidget( tracker );

    // tool bar
    toolBar = new QToolBar( this );
    CHECK_PTR(toolBar);
    QToolButton * addTb
	= new QToolButton( QPixmap(add_xpm), "New Project", QString::null,
			   this, SLOT(s_add()), toolBar, "add" );
    startTb
	= new QToolButton( QPixmap(play_xpm), "Start", QString::null,
			   this, SLOT(s_start()), toolBar, "start" );
    pauseTb
	= new QToolButton( QPixmap(pause_xpm), "Pause", QString::null,
			   this, SLOT(s_pause()), toolBar, "pause" );
    pauseTb->setEnabled(false);
    QToolButton * reportTb
	= new QToolButton( QPixmap(report_xpm), "Report", QString::null,
			   this, SLOT(s_report()), toolBar, "report" );    
    QToolButton * quitTb
	= new QToolButton( QPixmap(close_xpm), "Quit", QString::null,
			   this, SLOT(close()), toolBar, "quit" );

    // menu bar
    QPopupMenu *filemenu = new QPopupMenu();
    CHECK_PTR(filemenu);
    filemenu->insertItem("New Project...", this, SLOT(s_add()) );
    filemenu->insertItem("Delete Project", this, SLOT(s_delete()) );
    filemenu->insertSeparator();
    filemenu->insertItem("New Category...", this, SLOT(s_addcat()) );
    filemenu->insertItem( "Delete Category...", this, SLOT(s_delcat()) );    
    filemenu->insertSeparator();
    filemenu->insertItem( "Report", this, SLOT(s_report()));
    filemenu->insertSeparator();
    filemenu->insertItem("Exit", this, SLOT(close()) );
    menuBar()->insertItem("&File", filemenu);
    
    QPopupMenu *editmenu = new QPopupMenu();
    CHECK_PTR(editmenu);
    editmenu->insertItem("Project...", this, SLOT(s_edit()) );
    editmenu->insertItem("Preferences...", this, SLOT(s_pref()) );
    menuBar()->insertItem("&Edit", editmenu);

    viewmenu = new QPopupMenu();
    CHECK_PTR(viewmenu);
    viewmenu->setCheckable(true);
    viewmenu->insertItem("All", ID_ALL_VIEWMENU);
    // read category list from file
    QFile f(f_catlist);
    if ( f.open(IO_ReadOnly) ) {
        QTextStream ts( &f );
        QString s;
        while ( !ts.eof() ) {
	    s = ts.readLine().stripWhiteSpace();
	    cList.append(s);
	    viewmenu->insertItem(s, cList.count()-1 );
	}
        f.close();
    }
    viewmenu->insertItem("Unfiled", ID_UNFILED_VIEWMENU);
    menuBar()->insertItem("&View", viewmenu);
    viewmenu->setItemChecked( currentViewId, true );
    connect(viewmenu, SIGNAL( activated(int) ),
            this, SLOT( viewmenuCallBack(int) ) );
    
    QPopupMenu *clockmenu = new QPopupMenu();
    CHECK_PTR(clockmenu);
    clockmenu->insertItem("Start", this, SLOT(s_start()) );
    clockmenu->insertItem("Pause", this, SLOT(s_pause()) );
    clockmenu->insertSeparator();
    clockmenu->insertItem("Add 5 minutes", this, SLOT(s_plus()) );
    clockmenu->insertItem("Deduct 5 minutes", this, SLOT(s_minus()) );
    menuBar()->insertItem("&Clock", clockmenu);

    QPopupMenu *helpmenu = new QPopupMenu();
    CHECK_PTR(helpmenu);
    helpmenu->insertItem("About", this, SLOT(s_about()) );
    menuBar()->insertSeparator();
    menuBar()->insertItem("&Help", helpmenu);
    
    connect(tracker, SIGNAL(timerTick()), SLOT(updateTitle()) );
    connect(tracker, SIGNAL(changed()), SLOT(updateTitle() ) );
    connect(tracker, SIGNAL(autostopped()), SLOT(s_pause()) );
    connect(qApp, SIGNAL(aboutToQuit()), tracker, SLOT(dying()) );

    statusBar()->message("Taking a break");
    resize(pwidth, pheight);
}

Top::~Top()
{
    
}

void Top::updateTitle()
{
    if(tracker->isRunning()) {
	setCaption( buildText(captionFormat) );
	setIconText( buildText(iconFormat) );
	statusBar()->message("Working hard on " + tracker->getProject() );
    }
    else {
	setCaption( buildPausedText(captionFormat) );
	setIconText( buildPausedText(iconFormat) );
	statusBar()->message("Taking a break");
    }
}

void Top::s_add()
{
    AddDialog ad(cList);
    if (ad.exec()) {
        QString c;
        int i = ad.getCategoryInd();
        if(i == cList.count())  c = "";
        else  c = (cList[i]);
        tracker->newTask(ad.value(), c);
    }
}

void Top::s_delete()
{
    if(tracker->isRunning())
	statusBar()->message("Project running, cannot be deleted", 5000);
    else {
	if( QMessageBox::information ( this, "KTimeTracker: Delete Project",
                    "Are you sure you want\nto delete project: \n\n" +
                    tracker->getProject(),
                    QMessageBox::Yes, QMessageBox::No )
            == QMessageBox::Yes )

	    tracker->removeTask();
    }    
}

void Top::s_addcat()
{
    CatDialog cat;
    QString s;

    if(cat.exec()) {
	QString c = cat.value();

    if ( cList.contains( c  ) )
        return;
    cList.append(c);
	viewmenu->insertItem(c, cList.count()-1, viewmenu->count()-1);
	saveCategoryFile();
    }
}

void Top::s_delcat() 
{
    DelCatDialog dog( cList );
    if( dog.exec() ) {
	QString s = dog.getChosen().stripWhiteSpace();
	if( s.isEmpty() ) {
	    // nothing is chosen
	    statusBar()->message( "nothing is deleted", 2000 );
	}
	else {
	    // remove from cList
	    for(int i=0; i < cList.count(); i++) {
            if( cList[i] == s ) {
                cList.remove(s);
                int id = viewmenu->idAt(i+1);
                viewmenu->removeItem( id );    // update viewmenu
                break;
            }
	    }

	    // move all projects belonging to s to Unfiled
	    tracker->removeCategory( s );

	    saveCategoryFile();
	}
    }
}

void Top::s_edit()
{
    int time = tracker->getIntTime();
    int in = 0;
    if(time == -1)
	statusBar()->message("No project is selected",5000);
    else {
	QString sc = tracker->getCategory();
	for(in=0; in < cList.count(); in++) {
	    if(sc == (cList[in]))  break;   // last cat is "unfiled"
	}

	EditDialog ed(tracker->getProject(), in, time, cList);
	if(ed.exec()) {
	    if(ed.getTime() != time) {
		tracker->setTime(ed.getTime());
		updateTitle();
	    }
	    
	    if(ed.getCategoryInd() != in) {
		if(ed.getCategoryInd() < cList.count())
		    tracker->setCategory((cList[ed.getCategoryInd()]));
		else 
		    tracker->setCategory("");
	    }
	}
    }
}

void Top::s_plus()
{
    tracker->increment(5);
    updateTitle();
}

void Top::s_minus()
{
    tracker->decrement(5);
    updateTitle();
}

void Top::s_start()
{
    tracker->start();
    if(tracker->isRunning()) {
	startTb->setEnabled(false);
	pauseTb->setEnabled(true);
	updateTitle();
    }
    else
	statusBar()->message("Fail to start");
}

void Top::s_pause()
{
    tracker->pause();
    startTb->setEnabled(true);
    pauseTb->setEnabled(false);
    updateTitle();
}

void Top::s_pref()
{
    PrefDialog pd(p_savefreq, captionFormat, iconFormat, p_autostopTime);
    if(pd.exec()) {
	captionFormat = pd.getCaptionText();
	iconFormat = pd.getIconText();
	if(p_savefreq != pd.getFreq()) {
	    p_savefreq = pd.getFreq();
	    tracker->setSaveFreq(p_savefreq);
	}
	if(p_autostopTime != pd.getAutoStopTime()) {
	    p_autostopTime = pd.getAutoStopTime();
	    tracker->setAutostopTime(p_autostopTime);
	}

	updateTitle();
	savePrefFile();
    }
}

void Top::s_report()
{
    static Weeport *wee;
    static bool first = true;
    tracker->saveForReport();
    if(first) {
        wee = new Weeport();
        wee->show();
        first = false;
    } 
    else {
        wee->show();
    }
}

void Top::s_about()
{
    QMessageBox::about( this, "About KTimeTracker/Pi",
                        "KTimeTracker/Pi " + QString(VERSION) +"\n"
		"Copyright (c) 2004 Lutz Rogowski \n<lutz@pi-sync.net>\n"
		"KTimeTracker/Pi is licensed under the GPL\n\n"
		"KTimeTracker/Pi is based on:\n"
                        "PTimeTracker version 1.7.2"
	                "\n\nCopyright(c): 1999-2004\n"
			"Sylvia Wong <s.wong AT auckland.ac.nz>\n"
                        "happy <ljm AT ecs.soton.ac.uk>\n"
			"\nThis program comes with\nABSOLUTELY NO WARRANTY\n"
			"to the extent permitted by applicable law.");
}

void Top::viewmenuCallBack(int id)
{
    s_pause();  // pause timer when changing view. in future versions
                // this should be changed to: if current proj in new
                // view then continue, else pause 
    
    if(id == ID_ALL_VIEWMENU) {
	tracker->changeView("All");
	viewmenu->setItemChecked( currentViewId, false );
	currentViewId = ID_ALL_VIEWMENU;
	viewmenu->setItemChecked( currentViewId, true );
    }
    else if(id == ID_UNFILED_VIEWMENU) {
	tracker->changeView("Unfiled");
	viewmenu->setItemChecked( currentViewId, false );
	currentViewId = ID_UNFILED_VIEWMENU;
	viewmenu->setItemChecked( currentViewId, true );
    }
    else {
	tracker->changeView((cList[id]));
	viewmenu->setItemChecked( currentViewId, false );
	currentViewId = id;
	viewmenu->setItemChecked( currentViewId, true );
    }
	
}


/*
  things to do with the ptimetracker dir
*/
void Top::pttDirectory()
{
    QString pttdirvar = getenv("KTIMETRACKERDIR");
    if(pttdirvar.isEmpty())
        pttDir = QDir::homeDirPath() + "/kdepim/apps/ktimetracker/";
    else
	pttDir = pttdirvar + "/";    
    QDir dp(pttDir);
    if( !dp.exists() )
	if( !dp.mkdir(pttDir) ) {
	    QMessageBox::critical(0, "KTimeTracker: Fatal Error",
       	          QString("Cannot create directory ") + pttDir, "ok");
	    ::exit(1);
	}
}


/*
   Only 1 copy of this timetracker should be run at any one time
*/
void Top::lock()
{
    // on Zaurus, we can only start the app once. no lockfile needed
#ifdef DESKTOP_VERSION
    QFile lockfile(pttDir + "LOCK");
    
    if( lockfile.exists() ) {
	if ( lockfile.open(IO_ReadOnly) ) {
	    lockfile.close();
	    QMessageBox::critical( 0, "KTimeTracker: Fatal Error",
                  "A KTimeTracker may be\nalready running!\n"
                          "If you are sure\nthis is not the case, "
                	   "delete the\n" + lockfile.name() +"\nfile.", "OK");
	    
	}
	else
	    QMessageBox::critical( 0, "KTimeTracker: Fatal Error",
                 QString("An invalid lock file exist.\nDelete the\n") +
                 lockfile.name() +"\nfile and run this program again", "OK");
	::exit(1);
    }
    
    char* hostn = "localhost";
    if( lockfile.open(IO_WriteOnly) ) {
	QTextStream ts( &lockfile );
	ts << hostn << endl;
	lockfile.close();
    }
    else
	QMessageBox::warning(0, "KTimeTracker: Error",
	      QString("Failed to create lock file: ")+ lockfile.name(), "OK");
#endif
}

/*
  preference file
*/
void Top::readPrefFile()
{
    QFile preffile(pttDir + "preference");
    
    // defaults
    p_savefreq = DEFAULT_SAVE;
    captionFormat = DEFAULT_CAPTIONT;
    iconFormat = DEFAULT_ICONT;
    p_autostopTime = DEFAULT_AUTOSTOP;

    if( preffile.exists() ) {
	if ( preffile.open(IO_ReadOnly) ) {
	    QTextStream ts( &preffile );
	    int pos, i;
	    bool ok;
	    QString option, value, s;

	    while ( !ts.eof() ) {
		s = ts.readLine().stripWhiteSpace();
		if(s[0] == '#')  continue;  // skip comment lines
		
		pos = s.find('=');
		if(pos == -1)  continue;    // invalid line
		
		option = value = "";
		for(i=0; i<pos; i++)  option+= s[i];
		option = option.stripWhiteSpace();
		value = s.remove(0, pos+1);

		if(option == "autosave") {
		    i = value.toInt(&ok);
		    if(ok && i<=60 && i>=5) p_savefreq = i;
		}
		else if(option == "captionText") {
		    i = value.toInt(&ok);
		    if(ok && i<12 && i>=0) captionFormat = i;
		}
		else if(option == "iconText") {
		    i = value.toInt(&ok);
		    if(ok && i<12 && i>=0) iconFormat = i;
		}
		else if(option == "autostop") {
		    i = value.toInt(&ok);
		    if(ok && i<1440 && i>=-1)  p_autostopTime = i;
		}
	    }
	    preffile.close();
	}
    }
}

static const char *header = 
"# Configuration file for KTimeTracker\n"
"# Lines starting with '#' are comments\n";

static const char *autosaveComment =
"# Frequency of autosaving dayfile (between 5 to 60) in minutes\n"
"# Default is 30 minutes\n";

static const char *formatComment =
"# Format of caption and icon text\n"
"# 0 = T project time (default)\n# 1 = T time project\n"
"# 2 = T project\n# 3 = T time\n# 4 = project time\n# 5 = time project\n"
"# 6 = project\n# 7 = time\n# 8 = KTimeTracker: project time\n"
"# 9 = KTimeTrakcer: time project\n# 10 = KTimeTracker: project\n"
"# 11 = KTimeTracker: time\n";

static const char *autostopComment = 
"# Automatically stop the timer at this time of day\n"
"# (specified in number of minutes after midnight)\n"
"# autostop = 60  -> 1 am\n"
"# autostop = 1110 -> 6:30 pm (18*60 + 30)\n"
"# default is autostop = -1 (disabled)\n";

void Top::savePrefFile()
{
    QFile preffile(pttDir + "preference");
    
    if ( preffile.open(IO_WriteOnly) ) {
	QTextStream ts( &preffile );
	ts << header << "\n";
	ts << autosaveComment;
	if(p_savefreq == DEFAULT_SAVE)  
	    ts << "# autosave = 30\n\n";
	else 
	    ts << "autosave = " << p_savefreq << "\n";
	
	ts << "\n" << formatComment;
	if(captionFormat != DEFAULT_CAPTIONT)
	    ts <<"captionText = "<< captionFormat <<"\n";
	else 
	    ts << "# captionText = 0\n";
	if(iconFormat != DEFAULT_ICONT)  
	    ts <<"iconText = "<< iconFormat<<"\n";
	else 
	    ts << "# iconText = 0\n";
	
	ts << "\n" << autostopComment;
	if(p_autostopTime != DEFAULT_AUTOSTOP)
	    ts << "autostop = " << p_autostopTime << "\n";
	
	preffile.close();
    }
    else
	QMessageBox::warning(0, "KTimeTracker: Error",
	      QString("Cannot open preference file, ") + preffile.name() +
                      ", for writing", "OK");
}

/*
   read session file - size of window
*/
void Top::readSessionFile(int &pwidth, int &pheight)
{
    QFile f(pttDir + "session");
    
    // defaults
    pheight = 200;
    pwidth = 300;

    if( f.exists() && f.open(IO_ReadOnly) ) {
	QTextStream ts( &f );
	ts >> pwidth >> pheight;   // assume no error in reading !!
	f.close();
    }
}

/*
  build text shown in caption and icon according to format
*/
QString Top::buildText(int format)
{
    QString s;
    QString f1("T ");
    QString f2("KTimeTracker: ");
    
    switch(format){
    case 0: s = f1 + tracker->getProject() + " " + tracker->getTime(); break;
    case 1: s = f1 + tracker->getTime() + " " + tracker->getProject(); break;
    case 2: s = f1 + tracker->getProject(); break;
    case 3: s = f1 + tracker->getTime(); break;
    case 4: s = tracker->getProject() + " " + tracker->getTime(); break;
    case 5: s = tracker->getTime() + " " + tracker->getProject(); break;
    case 6: s = tracker->getProject(); break;
    case 7: s = tracker->getTime(); break;
    case 8: s = f2 + tracker->getProject() + " " + tracker->getTime(); break;
    case 9: s = f2 + tracker->getTime() + " " + tracker->getProject(); break;
    case 10: s = f2 + tracker->getProject(); break;
    case 11: s = f2 + tracker->getTime();
    }
    
    return s;
}

QString Top::buildPausedText(int format)
{
    QString s;
    QString f1("T ");
    QString f2("KTimeTracker: ");
    
    if(format < 4) s = f1 + "Paused";
    else if( format < 8 ) s = "Paused";
    else s = f2 + "Paused";    
    return s;
}


/*
    the category list
*/
void Top::saveCategoryFile()
{
    QFile f(f_catlist);
    if(f.open(IO_WriteOnly) ) {
	QTextStream ts( &f );
	for(int i=0; i < cList.count(); i++)
	    ts << cList[i].latin1() << "\n";
	f.close();
    }
    else
	QMessageBox::warning(0, "KTimeTracker: Error",
	   QString("Cannot save category file ") + f_catlist, "OK");
    
}
