/*
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999-2003 Sylvia Wong <s DOT wong AT auckland.ac.nz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/


#ifndef TOP_H
#define TOP_H

#include <qmainwindow.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qpopupmenu.h>
#include <qlist.h>
#include <qcheckbox.h>
#include <qlistbox.h>

class QToolBar;
class QToolButton;
class Sucker;
class QString;

#define DEFAULT_SAVE     30
#define DEFAULT_ICONT     0
#define DEFAULT_CAPTIONT  0
#define DEFAULT_AUTOSTOP -1
/************************************************************************
   add project dialog
*/

class AddDialog : public QDialog
{
public:
    AddDialog(QStringList cList);
    QString value() { return input->text(); }
    int getCategoryInd();  // category text selection (index)

private:
    QLineEdit *input;
    QComboBox *category_cb;
};

/************************************************************************
   add category dialog
*/

class CatDialog : public QDialog
{
public:
    CatDialog();
    QString value() { return input->text(); }

private:
    QLineEdit *input;
};

/************************************************************************
   delete category dialog
*/

class DelCatDialog : public QDialog
{
public:
    DelCatDialog( QStringList  cList );  // cList - list of categories
    QString getChosen();  // return currently highlighted item in "listbox"
    
private:
    QListBox *listbox;
};

/************************************************************************
   edit project dialog
*/
class EditDialog : public QDialog
{
public:
    EditDialog(QString pname, int pcat, int time, QStringList cList);
    int getTime();         // return time set in dialog in minutes
    int getCategoryInd();  // category text selection (index)
    
private:
    QSpinBox *hrbox, *minbox;
    QComboBox *category_cb;
};

/************************************************************************
   preference project dialog
*/
class PrefDialog : public QDialog
{
public:
    PrefDialog(int freq, int captiont, int icont, int autostopTime);
    int getFreq();         // return autosave freq in minutes
    int getCaptionText();  // caption text selection (index)
    int getIconText();     // icon text ....
    int getAutoStopTime(); // return time of day for autostopping
                           // in minutes after midnight
                           // -1 if the feature is not enabled
    
private:
    QSpinBox *freqbox, *autostop_hour, *autostop_min;
    QComboBox *caption_cb, *icon_cb;
    QCheckBox *autostop_cb;
};


/**********************************************************************
    Top Level Widget
*/
class Top: public QMainWindow
{
    Q_OBJECT
public:
    Top();
    ~Top();
    
public slots:
    void updateTitle();

private slots:
    void s_add();        // add new project, incl loading add dialog
    void s_delete();     // delete current project
    void s_edit();       // edit preference
    void s_start();      // start work on current project
    void s_pause();      // pause time tracker
    void s_pref();       // preference dialogue
    void s_plus();       // add 5 min to current project
    void s_minus();      // subtract 5 min
    void s_report();     // load report widget
    void s_about();      // about box
    void s_addcat();     // add new category
    void s_delcat();     // delete category
    void viewmenuCallBack(int);
    
private:
    QString  pttDir;      // store the ENV variable PTIMETRACKERDIR
    Sucker    *tracker;
    int captionFormat, iconFormat;
    QToolBar  *toolBar;
    QToolButton *startTb, *pauseTb;
    int p_savefreq;
    int p_autostopTime;    // time of day to autostop in min after midnight
    
    void lock();           // create lock for session
    void pttDirectory();   // stuff to do with the ptimetracker directory
    void readPrefFile();   // read "preference" file
    void savePrefFile();   // save ....
    void saveCategoryFile();   // save "categorylist" file

    void readSessionFile(int &pwidth, int &pheight);   
    // read "session" file, return width and height of main window in
    // last session
    
    QString buildText(int format); // build caption/icon text with 'format'
    QString buildPausedText(int format);  // similar but for paused

    QStringList cList;    // list of existing categories
    QString f_catlist;          // category list file name

    QPopupMenu *viewmenu;
    int currentViewId;       // current view as id to items on viewmenu
};


#endif



